@@ -10,6 +10,7 @@ import (
 	"os"
 	"os/exec"
 	"os/signal"
+	"os/user"
 	"path/filepath"
 	"runtime"
 	"strconv"
@@ -31,13 +32,14 @@ type Manager struct {
 	mu              *sync.Mutex
 	windowIndex     int
 	prevWindowIndex int
-	files           []file
+	prevDir         string
+	files           map[string]file
 	eventCh         chan<- event.Event
 	redrawCh        chan<- struct{}
 }
 
 type file struct {
-	name string
+	path string
 	file *os.File
 	perm os.FileMode
 }
@@ -50,14 +52,14 @@ func NewManager() *Manager {
 // Init initializes the Manager.
 func (m *Manager) Init(eventCh chan<- event.Event, redrawCh chan<- struct{}) {
 	m.eventCh, m.redrawCh = eventCh, redrawCh
-	m.mu = new(sync.Mutex)
+	m.mu, m.files = new(sync.Mutex), make(map[string]file)
 }
 
 // Open a new window.
-func (m *Manager) Open(filename string) error {
+func (m *Manager) Open(name string) error {
 	m.mu.Lock()
 	defer m.mu.Unlock()
-	window, err := m.open(filename)
+	window, err := m.open(name)
 	if err != nil {
 		return err
 	}
@@ -88,110 +90,105 @@ func (m *Manager) addWindow(window *window) {
 	m.windowIndex, m.prevWindowIndex = len(m.windows)-1, m.windowIndex
 }
 
-func (m *Manager) open(filename string) (*window, error) {
-	if filename == "" {
+func (m *Manager) open(name string) (*window, error) {
+	if name == "" {
 		window, err := newWindow(bytes.NewReader(nil), "", "", m.eventCh, m.redrawCh)
 		if err != nil {
 			return nil, err
 		}
 		return window, nil
 	}
-	if filename == "#" {
+	if name == "#" {
 		return m.windows[m.prevWindowIndex], nil
 	}
-	if strings.HasPrefix(filename, "#") {
-		index, err := strconv.Atoi(filename[1:])
+	if strings.HasPrefix(name, "#") {
+		index, err := strconv.Atoi(name[1:])
 		if err != nil || index <= 0 || len(m.windows) < index {
-			return nil, errors.New("invalid window index: " + filename)
+			return nil, errors.New("invalid window index: " + name)
 		}
 		return m.windows[index-1], nil
 	}
-	name, err := expandBacktick(filename)
+	name, err := expandBacktick(name)
 	if err != nil {
 		return nil, err
 	}
-	name, err = expandHomedir(name)
+	path, err := expandPath(name)
 	if err != nil {
 		return nil, err
 	}
-	filename = name
-	f, err := os.Open(filename)
+	r, err := m.openFile(path, name)
+	if err != nil {
+		return nil, err
+	}
+	return newWindow(r, path, filepath.Base(path), m.eventCh, m.redrawCh)
+}
+
+func (m *Manager) openFile(path, name string) (readAtSeeker, error) {
+	fi, err := os.Stat(path)
 	if err != nil {
 		if !os.IsNotExist(err) {
 			return nil, err
 		}
-		window, err := newWindow(bytes.NewReader(nil), filename, filepath.Base(filename), m.eventCh, m.redrawCh)
-		if err != nil {
-			return nil, err
-		}
-		return window, nil
+		return bytes.NewReader(nil), nil
+	} else if fi.IsDir() {
+		return nil, errors.New(name + " is a directory")
 	}
-	info, err := os.Stat(filename)
+	f, err := os.Open(path)
 	if err != nil {
 		return nil, err
 	}
-	if info.IsDir() {
-		return nil, errors.New(filename + " is a directory")
-	}
-	m.files = append(m.files, file{name: filename, file: f, perm: info.Mode().Perm()})
-	window, err := newWindow(f, filename, filepath.Base(filename), m.eventCh, m.redrawCh)
-	if err != nil {
-		return nil, err
-	}
-	return window, nil
+	m.addFile(path, f, fi)
+	return f, nil
 }
 
-func expandBacktick(filename string) (string, error) {
-	if !strings.HasPrefix(filename, "`") ||
-		!strings.HasSuffix(filename, "`") || len(filename) <= 2 {
-		return filename, nil
+func expandBacktick(name string) (string, error) {
+	if len(name) <= 2 || name[0] != '`' || name[len(name)-1] != '`' {
+		return name, nil
 	}
-	filename = strings.TrimSpace(filename[1 : len(filename)-1])
-	xs := strings.Fields(filename)
+	name = strings.TrimSpace(name[1 : len(name)-1])
+	xs := strings.Fields(name)
 	if len(xs) < 1 {
-		return filename, nil
+		return name, nil
 	}
 	out, err := exec.Command(xs[0], xs[1:]...).Output()
 	if err != nil {
-		return filename, err
+		return name, err
 	}
 	return strings.TrimSpace(string(out)), nil
 }
 
-func (m *Manager) read(r io.Reader) error {
-	done := make(chan struct{})
-	defer close(done)
-	abort := make(chan os.Signal, 1)
-	signal.Notify(abort, os.Interrupt)
-	defer signal.Stop(abort)
-	go func() {
-		select {
-		case <-time.After(time.Second):
-			fmt.Fprint(os.Stderr, "Reading stdin took more than 1 second, press <C-c> to abort...")
-		case <-done:
-		}
-	}()
-	bs, err := func() ([]byte, error) {
-		// ref: io.ReadAll
-		bs := make([]byte, 0, 1024)
-		for {
-			n, err := r.Read(bs[len(bs):cap(bs)])
-			bs = bs[:len(bs)+n]
+func expandPath(path string) (string, error) {
+	switch {
+	case strings.HasPrefix(path, "~"):
+		if name, rest, _ := strings.Cut(path[1:], string(filepath.Separator)); name != "" {
+			user, err := user.Lookup(name)
 			if err != nil {
-				if err == io.EOF {
-					err = nil
-				}
-				return bs, err
-			}
-			select {
-			case <-time.After(10 * time.Millisecond):
-			case <-abort:
-				return bs, nil
-			}
-			if len(bs) == cap(bs) {
-				bs = append(bs, 0)[:len(bs)]
+				return path, nil
 			}
+			return filepath.Join(user.HomeDir, rest), nil
 		}
+		homeDir, err := os.UserHomeDir()
+		if err != nil {
+			return "", err
+		}
+		return filepath.Join(homeDir, path[1:]), nil
+	case strings.HasPrefix(path, "$"):
+		name, rest, _ := strings.Cut(path[1:], string(filepath.Separator))
+		value := os.Getenv(name)
+		if value == "" {
+			return path, nil
+		}
+		return filepath.Join(value, rest), nil
+	default:
+		return filepath.Abs(path)
+	}
+}
+
+func (m *Manager) read(r io.Reader) error {
+	bs, err := func() ([]byte, error) {
+		r, stop := newReader(r)
+		defer stop()
+		return io.ReadAll(r)
 	}()
 	if err != nil {
 		return err
@@ -203,6 +200,38 @@ func (m *Manager) read(r io.Reader) error {
 	return m.init(window)
 }
 
+type reader struct {
+	io.Reader
+	abort chan os.Signal
+}
+
+func newReader(r io.Reader) (*reader, func()) {
+	done := make(chan struct{})
+	abort := make(chan os.Signal, 1)
+	signal.Notify(abort, os.Interrupt)
+	go func() {
+		select {
+		case <-time.After(time.Second):
+			fmt.Fprint(os.Stderr, "Reading stdin took more than 1 second, press <C-c> to abort...")
+		case <-done:
+		}
+	}()
+	return &reader{r, abort}, func() {
+		signal.Stop(abort)
+		close(abort)
+		close(done)
+	}
+}
+
+func (r *reader) Read(p []byte) (int, error) {
+	select {
+	case <-r.abort:
+		return 0, io.EOF
+	default:
+	}
+	return r.Reader.Read(p)
+}
+
 // SetSize sets the size of the screen.
 func (m *Manager) SetSize(width, height int) {
 	m.width, m.height = width, height
@@ -255,8 +284,9 @@ func (m *Manager) Emit(e event.Event) {
 		m.alternative(e)
 		m.eventCh <- event.Event{Type: event.Redraw}
 	case event.Wincmd:
-		if len(e.Arg) == 0 {
-			m.eventCh <- event.Event{Type: event.Error, Error: errors.New("an argument is required for " + e.CmdName)}
+		if e.Arg == "" {
+			m.eventCh <- event.Event{Type: event.Error,
+				Error: errors.New("an argument is required for " + e.CmdName)}
 		} else if err := m.wincmd(e.Arg); err != nil {
 			m.eventCh <- event.Event{Type: event.Error, Error: err}
 		} else {
@@ -328,16 +358,33 @@ func (m *Manager) Emit(e event.Event) {
 		} else {
 			m.eventCh <- event.Event{Type: event.Redraw}
 		}
+	case event.Pwd:
+		if e.Arg != "" {
+			m.eventCh <- event.Event{Type: event.Error, Error: errors.New("too many arguments for " + e.CmdName)}
+			break
+		}
+		fallthrough
+	case event.Chdir:
+		if dir, err := m.chdir(e); err != nil {
+			m.eventCh <- event.Event{Type: event.Error, Error: err}
+		} else {
+			m.eventCh <- event.Event{Type: event.Info, Error: errors.New(dir)}
+		}
 	case event.Quit:
 		if err := m.quit(e); err != nil {
 			m.eventCh <- event.Event{Type: event.Error, Error: err}
 		}
 	case event.Write:
-		if err := m.write(e); err != nil {
+		if name, n, err := m.write(e); err != nil {
 			m.eventCh <- event.Event{Type: event.Error, Error: err}
+		} else {
+			m.eventCh <- event.Event{Type: event.Info,
+				Error: fmt.Errorf("%s: %[2]d (0x%[2]x) bytes written", name, n)}
 		}
 	case event.WriteQuit:
-		if err := m.writeQuit(e); err != nil {
+		if _, _, err := m.write(e); err != nil {
+			m.eventCh <- event.Event{Type: event.Error, Error: err}
+		} else if err := m.quit(event.Event{Bang: e.Bang}); err != nil {
 			m.eventCh <- event.Event{Type: event.Error, Error: err}
 		}
 	default:
@@ -348,11 +395,9 @@ func (m *Manager) Emit(e event.Event) {
 func (m *Manager) edit(e event.Event) error {
 	m.mu.Lock()
 	defer m.mu.Unlock()
-	var name string
-	if len(e.Arg) == 0 {
-		name = m.windows[m.windowIndex].filename
-	} else {
-		name = e.Arg
+	name := e.Arg
+	if name == "" {
+		name = m.windows[m.windowIndex].path
 	}
 	window, err := m.open(name)
 	if err != nil {
@@ -364,7 +409,7 @@ func (m *Manager) edit(e event.Event) error {
 }
 
 func (m *Manager) enew(e event.Event) error {
-	if len(e.Arg) > 0 {
+	if e.Arg != "" {
 		return errors.New("too many arguments for " + e.CmdName)
 	}
 	m.mu.Lock()
@@ -395,7 +440,7 @@ func (m *Manager) newWindow(e event.Event, vertical bool) error {
 }
 
 func (m *Manager) only(e event.Event) error {
-	if len(e.Arg) > 0 {
+	if e.Arg != "" {
 		return errors.New("too many arguments for " + e.CmdName)
 	}
 	m.mu.Lock()
@@ -511,10 +556,39 @@ func (m *Manager) move(modifier func(layout.Window, layout.Layout) layout.Layout
 	}
 }
 
+func (m *Manager) chdir(e event.Event) (string, error) {
+	m.mu.Lock()
+	defer m.mu.Unlock()
+	if e.Arg == "-" && m.prevDir == "" {
+		return "", errors.New("no previous working directory")
+	}
+	dir, err := os.Getwd()
+	if err != nil {
+		return "", err
+	}
+	if e.Arg == "" {
+		return dir, nil
+	}
+	if e.Arg != "-" {
+		dir, m.prevDir = e.Arg, dir
+	} else {
+		dir, m.prevDir = m.prevDir, dir
+	}
+	if dir, err = expandPath(dir); err != nil {
+		return "", err
+	}
+	if err = os.Chdir(dir); err != nil {
+		return "", err
+	}
+	return os.Getwd()
+}
+
 func (m *Manager) quit(e event.Event) error {
-	if len(e.Arg) > 0 {
+	if e.Arg != "" {
 		return errors.New("too many arguments for " + e.CmdName)
 	}
+	m.mu.Lock()
+	defer m.mu.Unlock()
 	window := m.windows[m.windowIndex]
 	if window.changedTick != window.savedChangedTick && !e.Bang {
 		return errors.New("you have unsaved changes in " + window.getName() + ", add ! to force :quit")
@@ -523,38 +597,76 @@ func (m *Manager) quit(e event.Event) error {
 	if w == 1 && h == 1 {
 		m.eventCh <- event.Event{Type: event.QuitAll}
 	} else {
-		m.mu.Lock()
 		m.layout = m.layout.Close().Resize(0, 0, m.width, m.height)
 		m.windowIndex, m.prevWindowIndex = m.layout.ActiveWindow().Index, m.windowIndex
-		m.mu.Unlock()
 		m.eventCh <- event.Event{Type: event.Redraw}
 	}
 	return nil
 }
 
-func (m *Manager) write(e event.Event) error {
+func (m *Manager) write(e event.Event) (string, int64, error) {
 	if e.Range != nil && e.Arg == "" {
-		return errors.New("cannot overwrite partially with " + e.CmdName)
+		return "", 0, errors.New("cannot overwrite partially with " + e.CmdName)
 	}
-	filename, n, err := m.writeFile(e.Range, e.Arg)
+	m.mu.Lock()
+	defer m.mu.Unlock()
+	window := m.windows[m.windowIndex]
+	var path string
+	name := e.Arg
+	if name == "" {
+		if window.name == "" {
+			return "", 0, errors.New("no file name")
+		}
+		path, name = window.path, window.name
+	} else {
+		var err error
+		path, err = expandPath(name)
+		if err != nil {
+			return "", 0, err
+		}
+	}
+	if runtime.GOOS == "windows" && m.opened(path) {
+		return "", 0, errors.New("cannot overwrite the original file on Windows")
+	}
+	if window.path == "" && window.name == "" {
+		window.setPathName(path, filepath.Base(path))
+	}
+	tmpf, err := os.OpenFile(
+		path+"-"+strconv.FormatUint(rand.Uint64(), 36),
+		os.O_RDWR|os.O_CREATE|os.O_EXCL, m.filePerm(path),
+	) //#nosec G404
 	if err != nil {
-		return err
+		return "", 0, err
 	}
-	m.eventCh <- event.Event{Type: event.Info, Error: fmt.Errorf("%s: %d (0x%x) bytes written", filename, n, n)}
-	return nil
+	defer os.Remove(tmpf.Name())
+	n, err := window.writeTo(e.Range, tmpf)
+	if err != nil {
+		_ = tmpf.Close()
+		return "", 0, err
+	}
+	if err = tmpf.Close(); err != nil {
+		return "", 0, err
+	}
+	if window.path == path {
+		window.savedChangedTick = window.changedTick
+	}
+	return name, n, os.Rename(tmpf.Name(), path)
 }
 
-func (m *Manager) writeQuit(e event.Event) error {
-	if len(e.Arg) > 0 {
-		return errors.New("too many arguments for " + e.CmdName)
+func (m *Manager) addFile(path string, f *os.File, fi os.FileInfo) {
+	m.files[path] = file{path: path, file: f, perm: fi.Mode().Perm()}
+}
+
+func (m *Manager) opened(path string) bool {
+	_, ok := m.files[path]
+	return ok
+}
+
+func (m *Manager) filePerm(path string) os.FileMode {
+	if f, ok := m.files[path]; ok {
+		return f.perm
 	}
-	if e.Range != nil {
-		return errors.New("range not allowed for " + e.CmdName)
-	}
-	if _, _, err := m.writeFile(nil, ""); err != nil {
-		return err
-	}
-	return m.quit(e)
+	return os.FileMode(0o644)
 }
 
 // State returns the state of the windows.
@@ -581,78 +693,9 @@ func hexWindowWidth(width int) int {
 	return width & (0b11 << (bits.Len(uint(width)) - 2))
 }
 
-func (m *Manager) writeFile(r *event.Range, name string) (string, int64, error) {
-	window := m.windows[m.windowIndex]
-	if name == "" {
-		name = window.filename
-	}
-	if name == "" {
-		return name, 0, errors.New("no file name")
-	}
-	if runtime.GOOS == "windows" && m.opened(name) {
-		return name, 0, errors.New("cannot overwrite the original file on Windows")
-	}
-	var err error
-	if name, err = expandHomedir(name); err != nil {
-		return name, 0, err
-	}
-	if window.filename == "" && window.name == "" {
-		window.mu.Lock()
-		window.filename = name
-		window.name = filepath.Base(name)
-		window.mu.Unlock()
-	}
-	tmpf, err := os.OpenFile(
-		name+"-"+strconv.FormatUint(rand.Uint64(), 16),
-		os.O_RDWR|os.O_CREATE|os.O_EXCL, m.filePerm(name),
-	) //#nosec G404
-	if err != nil {
-		return name, 0, err
-	}
-	defer os.Remove(tmpf.Name())
-	n, err := window.writeTo(r, tmpf)
-	tmpf.Close()
-	if err != nil {
-		return name, 0, err
-	}
-	if window.filename == name {
-		window.savedChangedTick = window.changedTick
-	}
-	return name, n, os.Rename(tmpf.Name(), name)
-}
-
-func (m *Manager) filePerm(name string) os.FileMode {
-	for _, f := range m.files {
-		if f.name == name {
-			return f.perm // keep the permission of the original file
-		}
-	}
-	return os.FileMode(0o644)
-}
-
-func (m *Manager) opened(name string) bool {
-	for _, f := range m.files {
-		if f.name == name {
-			return true
-		}
-	}
-	return false
-}
-
 // Close the Manager.
 func (m *Manager) Close() {
 	for _, f := range m.files {
-		f.file.Close()
+		_ = f.file.Close()
 	}
 }
-
-func expandHomedir(path string) (string, error) {
-	if !strings.HasPrefix(path, "~") {
-		return path, nil
-	}
-	homeDir, err := os.UserHomeDir()
-	if err != nil {
-		return "", err
-	}
-	return filepath.Join(homeDir, path[1:]), nil
-}
